<?php

require_once dirname(__FILE__) . '/app.php';

// quick checkout: /?page_id=8&add-to-cart=11&quantity=1&payment=bankpay
// curl -X POST "http://localhost:8000/?woo-api=callback_oppwa&order=xxx&checkoutId=xxx"

if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) )
{
	function  oppwa_api_callback()
	{
		global $woocommerce;

		$logs = new WC_Logger();

		if (empty($_REQUEST['order']) && !empty($_REQUEST['order']) && $_REQUEST['woo-api'] === 'callback_oppwa') {
			$logs->add( 'oppwa', __( 'OPPWA plugin: request parameter `order` is missing ' . print_r($_REQUEST, true), 'woocommerce') );
			return false;
		} else if (empty($_REQUEST['order'])) {
			return false;
		}

		$order_id = (!empty($_REQUEST['order']) ? filter_var($_REQUEST['order'], FILTER_SANITIZE_NUMBER_INT) : '' );
		$order = new WC_Order( $order_id );
		$order_data = $order->get_data();

		if ($order_id >= 1 && !empty($order_data)) {

			$gateways = $woocommerce->payment_gateways->payment_gateways();

			if ( $_REQUEST['woo-api'] != 'callback_oppwa' ) {
				$logs->add( 'oppwa', __('OPPWA plugin: Invalid Callback URL', 'woocommerce') );
				return;
			}

			if ( !isset($gateways['oppwa']) ) {
				$logs->add( 'oppwa', __('OPPWA plugin: not enabled in woocommerce', 'woocommerce') );
				return;
			}

			try {
				$logs->add( 'oppwa', __( 'OPPWA plugin: check payment status Order #' . $order_id, 'woocommerce') );

				$oppwa = $gateways['oppwa'];
				$checkout = (object) [];
				$checkout->id = $_REQUEST['id'];

				// get payment status
				$transaction = paymentStatus($checkout);

				$logs->add( 'oppwa', __( 'OPPWA plugin: check response Order: ' . $order_id . ' | Checkout: ' . $checkout->id . ' - ' . print_r($transaction, true), 'woocommerce') );

				if (!isset($transaction->result)) {
					throw new Exception('Transaction not found: ' . $checkout->id);
				}

				$successStatusCodesRegEx = '/^(000.000.|000.100.1|000.[36]|000.400.[1][12]0)/';
				$reviewStatusCodesRegEx = '/^(000.400.0[^3]|000.400.100)/';
				preg_match($successStatusCodesRegEx, $transaction->result->code, $successMatches);
				preg_match($reviewStatusCodesRegEx, $transaction->result->code, $reviewMatches);

				// $testSuccessCode = '000.100.110'
				$successCode = (is_array($successMatches) && !empty($successMatches[1]));
				$reviewCode = (is_array($reviewMatches) && !empty($reviewMatches[1]));

				$isStatusClosed = (isset($transaction->paymentBrand) && ($successCode === true || $reviewCode === true));

			    if ($isStatusClosed) {
				    $order->payment_complete();

					if ($successCode === true) {
						$order->add_order_note(
							sprintf(
								"%s Payment completed and confirmed by the OPPWA PSP <a href='https://docs.oppwa.com/#checkout/%s'>%s</a> (%s)",
								$oppwa->method_title . ' | ' . $transaction->paymentBrand,
								$transaction->id,
								$transaction->result->description,
								$transaction->result->code,
							)
						);
					} else {
						$order->add_order_note(
							sprintf(
								"%s Payment completed - Review OPPWA PSP confirmation <a href='https://docs.oppwa.com/#checkout/%s#%s'>%s</a> (%s)",
								$oppwa->method_title . ' | ' . $transaction->paymentBrand,
								$transaction->id,
								$transaction->result->description,
								$transaction->result->code,
							)
						);
					}

					$woocommerce->cart->empty_cart();

					$redirect_url = $order->get_view_order_url();

					/*
					// TODO activate later
					if ($oppwa->returnUrl == '' || $oppwa->returnUrl == 0 ) {
						$redirect_url = $oppwa->get_return_url( $order );
					} else {
						$redirect_url = get_permalink( $oppwa->returnUrl );
					}
					*/

					wp_redirect( $redirect_url );
					exit;

			    } else {

			    	$order->add_order_note(
			            sprintf(
			                "%s Payment failed. Order-ID: %s <a href='https://docs.oppwa.com/#checkout/%s'>%s</a> (%s)",
			                $oppwa->method_title . ' | ' . $transaction->paymentBrand,
							$order_id,
			                $transaction->id,
			                $transaction->result->description,
							$transaction->result->code,
			            )
			        );

			        wc_add_notice(__( 'Transaction Error: Could not complete your payment - Order-ID: ' . $order_id . ' (Payment: ' . $transaction->id . ' | Code: ' . $transaction->result->code . ': ' . $transaction->result->description . ')', 'woocommerce'), 'error');
			        wp_redirect( wc_get_checkout_url() ); exit;
			    }
			} catch ( Exception $e ) {

				$trxnresponsemessage = $e->getMessage();
				$order->add_order_note(
			            sprintf(
			                "%s Payment Failed with message: '%s'",
			                $oppwa->method_title,
			                $trxnresponsemessage
			            )
			        );

				wc_add_notice(__( 'Transaction Error: Could not complete your payment' , 'woocommerce'), 'error');
		  		wp_redirect( wc_get_checkout_url() ); exit;
			}
		} else {
			$logs->add( 'oppwa', __( 'OPPWA plugin: `order` not found with given ID: ' + print_r($_REQUEST['order'], true), 'woocommerce') );
			return false;
		}
	}

	add_action( 'init', 'oppwa_api_callback' );

	/**
	 * Perform HTTP request to REST endpoint
	 *
	 * @param string $action
	 * @param array  $params
	 * @param string $privateApiKey
	 *
	 * @return array
	 */
	function oppwaRequestApi($action = '', $params = array(), $privateApiKey)
	{
		global $oppwa;

		$logs = new WC_Logger();

		$args = array(
			'body'        => $params,
			'timeout'     => '5',
			'redirection' => '5',
			'httpversion' => '1.0',
			'blocking'    => true,
			'headers'     => array(),
			'cookies'     => array(),
		);

		// TODO: / or no /
		$response = wp_remote_post( $oppwa->apiEndpoint . '/' . $action, $args );

		if (is_wp_error($response)) {
			if ($logs) $logs->add( 'oppwa', __( 'OPPWA plugin: API error: ' . print_r($response->get_error_messages(), true), 'woocommerce') );

			return array(
				'header' => array(
					'status' => 500,
					'reason' => null,
				),
				'body' => array('error' => print_r($response->get_error_messages(), true)),
			);
		} else {
			return array(
				'header' => array(
					'status' => 200,
					'reason' => null,
				),
				'body' => $response['body'],
			);
		}
	}

	/**
	 * Perform API and handle exceptions
	 *
	 * @param        $action
	 * @param array  $params
	 * @param string $privateApiKey
	 *
	 * @return mixed
	 */
	function oppwaRequest($action, $params = array(), $privateApiKey)
	{
		if (!is_array($params)) {
			$params = array();
		}

		$logs = new WC_Logger();

		// $logs->add( 'oppwa', __( 'OPPWA plugin: API Request params: ' . $action . ' -- ' . json_encode($params), 'woocommerce') );

		$responseArray = oppwaRequestApi($action, $params, $privateApiKey);
		$httpStatusCode = $responseArray['header']['status'];

		// $logs->add( 'oppwa', __( 'OPPWA plugin: API Request response: ' . $action . ' -- ' . json_encode($responseArray), 'woocommerce') );

		if ($httpStatusCode != 200) {
			$errorMessage = 'Client returned HTTP status code ' . $httpStatusCode;
			if (isset($responseArray['body']['error'])) {
				$errorMessage = $responseArray['body']['error'];
			}

			$responseCode = '';
			if (isset($responseArray['body']['data']['response_code'])) {
				$responseCode = $responseArray['body']['data']['response_code'];
			}

			return array("data" => array(
				"error"            => $errorMessage,
				"response_code"    => $responseCode,
				"http_status_code" => $httpStatusCode
			));
		}

		return (array) json_decode($responseArray['body']);
	}
}

/*
// test card: 4242 4242 4242 4242
// response:

stdClass Object
(
    [id] => 8ac7a4a286ef02d90186ef1f754c6797
    [paymentType] => DB
    [paymentBrand] => VISA
    [amount] => 0.42
    [currency] => EUR
    [descriptor] => 4781.2734.8234 OPP_Channel
    [result] => stdClass Object
        (
            [code] => 000.100.110
            [description] => Request successfully processed in 'Merchant in Integrator Test Mode'
        )

    [resultDetails] => stdClass Object
        (
            [clearingInstituteName] => Elavon-euroconex_UK_Test
        )

    [card] => stdClass Object
        (
            [bin] => 424242
            [last4Digits] => 4242
            [holder] => Foo Bar
            [expiryMonth] => 12
            [expiryYear] => 2024
            [type] => CREDIT
            [level] => CLASSIC
            [country] => GB
            [maxPanLength] => 16
            [binType] => PERSONAL
            [regulatedFlag] => N
        )

    [customer] => stdClass Object
        (
            [ip] => 81.217.141.51
            [ipCountry] => AT
            [browserFingerprint] => stdClass Object
                (
                    [value] => 0400GP3AxROZu5CVebKatfMjIEml1wJFZNvfR7fvTeXztIgLvW51nhGGCqt6heU6JIInFuZER9PSyOH4+ ...
                )

        )

    [threeDSecure] => stdClass Object
        (
            [eci] => 07
        )

    [customParameters] => stdClass Object
        (
            [SHOPPER_EndToEndIdentity] => c2b5260b6c3ea0c2f0c4a355c15c4ad3693e397f3ef463ea30dad2aa293720e8
            [CTPE_DESCRIPTOR_TEMPLATE] =>
            [FEEDZAI_DATA_FEED] => true
        )

    [risk] => stdClass Object
        (
            [score] => 0
        )

    [buildNumber] => 7753f97b6133c272bc47c5f7b02d6da27f1815c1@2023-03-16 10:35:57 +0000
    [timestamp] => 2023-03-17 10:31:01.880+0000
    [ndc] => 2EEE06D7DB63BF914040923CD4763FA9.uat01-vm-tx01
)
*/
