<?php

// quick-checkout: /?page_id=8&add-to-cart=11&payment=oppwa

require_once 'app.php';

function woocommerce_api_oppwa_init() {

	if (!class_exists('WC_Payment_Gateway')) return;

	/**
	 * OPPWA OPPWA Payment Gateway.
	 *
	 * Provides a OPPWA OPPWA Payment Gateway. Based on code by Mike Pepper.
	 *
	 * @class       WC_Gateway_oppwa
	 * @extends     WC_Payment_Gateway
	 * @version     1.1.5
	 */
	class WC_Gateway_oppwa extends WC_Payment_Gateway {

		/**
		 * Array of locales
		 *
		 * @var array
		 */
		public $locale;

		/**
		 * Constructor for the gateway.
		 */
		public function __construct() {

			$this->id                 = 'oppwa';
			$this->icon               = apply_filters( 'woocommerce_oppwa_icon', '' );
			$this->has_fields         = false;
			$this->method_title       = __( 'OPPWA', 'woocommerce' );
			$this->method_description = __( 'Accept payments via OPPWA', 'woocommerce' );

			// Load the settings.
			$this->init_form_fields();
			$this->init_settings();

			// Define user set variables.
			$this->title        = $this->get_option( 'title' );
			$this->description  = $this->get_option( 'description' );
			$this->instructions = $this->get_option( 'instructions' );

			$this->mode 			= $this->settings[ 'mode' ];
			$this->returnUrl 		= $this->settings[ 'returnUrl' ];
			$this->debugMode  		= $this->settings[ 'debugMode' ];
			$this->notify_url   	= add_query_arg( array(
														'woo-api' => 'callback_oppwa',
													), home_url( '/' ));
			$this->msg['message'] 	= '';
			$this->msg['class'] 	= '';

			if ( $this->debugMode == 'on' ) {
				$this->logs = new WC_Logger();
			}

			// Actions
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( &$this, 'process_admin_options' ) );
			add_action( 'woocommerce_thankyou_oppwa', array( &$this, 'thankyou_page' ) );
			add_action( 'woocommerce_receipt_oppwa', array( &$this, 'receipt_page' ) );

			// Customer Emails
			// TODO
			add_action( 'woocommerce_email_before_order_table', array( &$this, 'email_instructions' ), 10, 3 );
		}

		/**
		 * Initialise Gateway Settings Form Fields.
		 */
		public function init_form_fields() {

			$this->form_fields = array(
				'enabled'         => array(
					'title'   => __( 'Enable/Disable', 'woocommerce' ),
					'type'    => 'checkbox',
					'label'   => __( 'Enable OPPWA', 'woocommerce' ),
					'default' => 'no',
				),
				'merchantName' => array(
					'title' 		=> __( 'Company', 'woocommerce' ),
					'type' 			=> 'text',
					'description' 	=> __( 'Company name will be used at the checkout.', 'woocommerce' ),
					'required' 		=> true,
					'desc_tip'      => true,
				),
				'title'           => array(
					'title'       => __( 'Title', 'woocommerce' ),
					'type'        => 'text',
					'description' => __( 'This controls the title which the user sees during checkout.', 'woocommerce' ),
					'default'     => __( 'Apple Pay™ | Google Pay™', 'woocommerce' ),
					'desc_tip'    => true,
				),
				'description'     => array(
					'title'       => __( 'Description', 'woocommerce' ),
					'type'        => 'textarea',
					'description' => __( 'Payment method description that the customer will see on your checkout.', 'woocommerce' ),
					'default'     => __( 'Make your payment directly and securely with Apple | Google Pay or Debit | Credit cards.', 'woocommerce' ),
					'desc_tip'    => true,
				),
				'instructions'    => array(
					'title'       => __( 'Instructions', 'woocommerce' ),
					'type'        => 'textarea',
					'description' => __( 'Information that will be added to the thank you page and emails.', 'woocommerce' ),
					'default'     => '',
					'desc_tip'    => true,
				),
				'debugMode' => array(
					'title' 		=> __( 'Debug Mode', 'woocommerce' ),
					'type' 			=> 'select',
					'description' 	=> '',
					'options'     	=> array(
						'off' 		=> __( 'Off', 'woocommerce' ),
						'on' 		=> __( 'On', 'woocommerce' )
					)
				)
			);
		}

		/**
		 * Admin Options.
		 */
		public function admin_options()
		{
			if ($this->mode == 'p' && get_option('woocommerce_force_ssl_checkout') == 'no' && $this->enabled == 'yes') {
				// echo '<div class="error"><p>'.sprintf(__('%s Sandbox testing is disabled and can performe live transactions but the <a href="%s">force SSL option</a> is disabled; your checkout is not secure! Please enable SSL and ensure your server has a valid SSL certificate.', 'woothemes'), $this->method_title, admin_url('admin.php?page=wc-settings&tab=checkout')).'</p></div>';
			}

			// TODO
			$currencies = array("EUR");

			if (!in_array(get_option('woocommerce_currency'), $currencies )) {
				// echo '<div class="error"><p>'.__('OPPWA currently only supports EUR.', 'woocommerce').'</p></div>';
			}

			echo '<h3>'.__(	'OPPWA Payment Gateway', 'woocommerce' ).'</h3>';

			echo '<table class="form-table">';
			$this->generate_settings_html();
			echo '</table>';
		}

		/**
		 * Output for the order receipt page.
		 *
		 * @param int $order_id Order ID.
		 */
		public function receipt_page( $order_id ) {

			/*echo '<p>'.__(
				sprintf(
						"Apple Pay™ | Google Pay™",
						$this->method_title
						), 'woocommerce' ).'</p>';*/

			echo $this->set_payment_form( $order_id );

			if ( $this->mode == 's' ) {
				echo '<p>';
				echo wpautop( wptexturize(  __('TEST MODE/SANDBOX ENABLED', 'woocommerce') )). ' ';
				echo '<p>';
			}
		}

		/**
		 * Output for the order form.
		 *
		 * @param int $order_id Order ID.
		 */
		public function set_payment_form( $order_id = false) {

			global $oppwa;

			$order_total = $this->get_order_total();
			// $order_id = $this->get_order_id();

			if (!$order_id) {
				$order_id = absint( get_query_var( 'order-pay' ) );
			}

			$order = new WC_Order( $order_id );
			// $host = @parse_url(strtolower(get_bloginfo('atom_url')), PHP_URL_HOST);

			$data = $order->get_data();
			$order_id = $data['id'];

			if (empty($order_id)) {
				// return '<div class="error"><p>'.sprintf(__('OPPWA Checkout Fehler - Keine Bestellung gefunden', 'woothemes')).'</p></div>';
				$reference = '';
			} else {
				$reference = "Order #$order_id-" . $data['date_created']->date('Ymd');
			}

			if (!$pay || !isset($pay->amount)) {
				$pay = (object) [
					"amount" => "0",
					"currency" => "EUR"
				];
			}

			if ($order_total >= 0) {
				$pay->amount = $order_total;
			}

			if ($data['payment_method'] == 'oppwa'
				&& empty($data['oppwa_transaction_id'])) {

					if (empty($pay->currency)) {
						$pay->currency = 'EUR';
					}

					$url = $oppwa->apiEndpoint . "/checkouts";


					if ($this->logs) $this->logs->add( 'oppwa', __( 'OPPWA plugin: init TX: ' . print_r($data, true), 'woocommerce') );
					// echo '<div class="error"><p>'.sprintf(__('OPPWA Update Fehler - set payment form - <a href="%s">Error-Log anzeigen</a>', 'woothemes'), admin_url('admin.php?page=wc-status&tab=logs')).'</p></div>';

					if (!empty($data['total'])) {
						$pay->amount = $data['total'];
					}

					$post_data = array(
						'entityId' => $oppwa->clientId,
						'amount' => $pay->amount,
						'currency' => $pay->currency,
						'paymentType' => 'DB', // PA
					);

					$authorization = array('Authorization' => 'Bearer ' . $oppwa->authToken);

					$args = array(
						'body'        => $post_data,
						'timeout'     => '5',
						'redirection' => '5',
						'httpversion' => '1.0',
						'blocking'    => true,
						'headers'     => $authorization,
						'cookies'     => array(),
					);

					$response = wp_remote_post( $url, $args );

					if (is_wp_error($response)) {
						if ($this->logs) $this->logs->add( 'oppwa', __( 'OPPWA plugin: checkout error: ' . print_r($response->get_error_messages(), true), 'woocommerce') );
						return '<div class="error"><p>'.sprintf(__('OPPWA Checkout Fehler - <a href="%s">Error-Log anzeigen</a>', 'woothemes'), admin_url('admin.php?page=wc-status&tab=logs')).'</p></div>';
					} else {
						$checkout = (object) json_decode($response['body']);
					}

					if (!empty($checkout->id) && $order_id > 0) {
						add_post_meta($order_id, '_oppwa_transaction_id', $checkout->uuid);
						add_post_meta($order_id, '_oppwa_checkout_id', $checkout->uuid);

						$checkoutId = $checkout->id;
					} else if (empty($checkout->id)) {
						print_r($checkout, true);
						if ($this->logs) $this->logs->add( 'oppwa', __( 'OPPWA plugin: checkout error: ' . print_r($checkout, true), 'woocommerce') );
					}

					$checkoutId = $checkout->id;
					$checkoutMsg = $checkout->result->description;
			} else if ($data['payment_method'] == 'oppwa'
				&& !empty($data['oppwa_transaction_id'])) {
				$checkoutId = $data['oppwa_transaction_id'];
				$checkoutMsg = 'TX loaded';
			} else {
				// create transaction
				$checkout = paymentInitCheckout((object) ['amount' => $order_total]);
				$checkoutId = $checkout->id;
				$checkoutMsg = $checkout->result->description;
			}

			$checkoutMsg .= ' <br /> # ' . $checkoutId;
			// . ' Amount: ' . $order_total;


			$html = '<script>';

			$html .= '
				var wpwlOptions = {
					style: "logos",
					imageStyle: "svg",

					brandDetection: true,
					brandDetectionType: "binlist",
					brandDetectionPriority: ["APPLEPAY", "GOOGLEPAY", "MASTER", "VISA"],

					applePay: {
						displayName: "' . $oppwa->apay->displayName . '",
						total: {
							label: "' . $oppwa->apay->displayName . '"
						}
					},

					googlePay: {
						// Channel Entity ID
						gatewayMerchantId: "' . $oppwa->gpay->gatewayMerchantId . '",
					},

					onReady: function() {
						console.log(
							"OPPWA WooCommerce Plugin: ready for payment"
						);

						ready = true;

						// $(".wpwl-group-brand").hide();
						$(".wpwl-group-expiry").hide();
						$(".wpwl-group-cardHolder").hide();
						$(".wpwl-group-cvv").hide();
						$(".wpwl-group-submit").hide();

						showBrands();
					},

					onChangeBrand: function() {
						console.log(
							"OPPWA WooCommerce Plugin: payment brand changed"
						);

						hideBrands();
					}
				};

				var ready = false;

				function showBrands() {
					// $(".wpwl-group-brand").show();
					// $(".wpwl-group-card-logos-horizontal > div:is(.wpwl-brand-card)").addClass("wpwl-hidden")

					window.setTimeout(function() {
						// show visa and mastercard
						/*
						$(".wpwl-group-card-logos-horizontal > div.wpwl-brand-card-logo-highlighted").removeClass("wpwl-brand-card-logo-highlighted");
						$(".wpwl-group-card-logos-horizontal > div[value=\'VISA\']").removeClass("wpwl-hidden").show();
						$(".wpwl-group-card-logos-horizontal > div[value=\'MASTER\']").removeClass("wpwl-hidden").show();
						// */
					}, 1991);
				}

				function hideBrands() {
					if (!ready /*|| dotsClicked*/ ) {
						return;
					}

					/*
					$(".wpwl-group-card-logos-horizontal > div[value=\'VISADEBIT\']").addClass("wpwl-hidden").hide();
					$(".wpwl-group-card-logos-horizontal > div[value=\'MASTERDEBIT\']").addClass("wpwl-hidden").hide();

					$(".wpwl-group-brand").show();

					$(".wpwl-group-expiry").show();
					$(".wpwl-group-cardHolder").show();
					$(".wpwl-group-cvv").show();
					$(".wpwl-group-submit").show();

					// Clears all previous dots-hidden logos, if any
					// $(".wpwl-group-card-logos-horizontal > div").removeClass("dots-hidden");

					// Selects all non-hidden logos. They are detected brands which otherwise would be shown by default.
					// var $logos = $(".wpwl-group-card-logos-horizontal > div:is(.wpwl-brand-card)");
					/*var $logos = $(".wpwl-group-card-logos-horizontal > div:not(.wpwl-hidden)");
					var $isSelected = $(".wpwl-group-card-logos-horizontal > div:is(.wpwl-brand-card-logo-highlighted)");

					if ($isSelected.length < 1) {
					}

					if ($($logos).size() < 2) {
						return;
					}

					$(".wpwl-group-card-logos-horizontal").css("width", "100%");

					// Hides all except the first logo, and displays three dots (...)
					$logos.first().after($("<div>...</div>").addClass("dots"));
					$logos.filter(function(index) { return index > 0; }).addClass("dots-hidden");

					// If ... is clicked, un-hides the logos
					$(".dots").click(function() {
						dotsClicked = true;
						$(".dots-hidden").removeClass("dots-hidden").show();
						$(this).remove();
					});
					*/
				}
			</script>

			<script src="' . $oppwa->apiEndpoint . '/paymentWidgets.js?checkoutId=' . $checkoutId .'"></script>';

			$html .= '<style>
				ul.order_details {
					max-width: 50%;
					min-width: 40%;
					float: left;
				}

				ul.order_details li.date, ul.order_details li.method {
					display: none;
				}

				#oppwa-payment {
					margin: 1.5em auto;
					padding: 0.5em;

					max-width: 50%;
					min-width: 50%;

					__min-height: 222px;
					__max-height: 999px;
					__min-width: 240px;
					__max-width: 540px;

					font-family: Arial, Helvetica, sans-serif;
					font-size: 1.1em;

					float: right;

					text-align: center;
				}

				#oppwa-payment-message {
					text-align: center;
					font-size: 0.5em;
					padding-top: 1.5em;
				}

				.wpwl-button-pay {
					width: 100%;
				}

				.wpwl-message {
					font-size: 0.8em;
					line-height: 1.5em;
				}

				.wpwl-message span {
					font-size: 0.9em;
					font-weight: bold;
					padding: 1.5em;
					display: block;
				}

				.wpwl-group-button div {
					margin: 0.1em auto;
					width: 240px;
				}

				/*
				.dots { margin-left: 4px; }
				.dots-hidden { display: none; }
				*/
			</style>';

			$html .= '<div id="oppwa-payment">
				<div id="oppwa-payment-cards">
					<form action="' . $oppwa->checkoutResultUrl . '&order=' . $order_id . '" class="paymentWidgets" data-brands="' . $oppwa->brands . '"></form>
				</div>
				<p id="oppwa-payment-message">' . $checkoutMsg .'</p>
			</div>';

			$html .= '<script>console.log("WC OPPWA Checkout", "' . $order_id . ' | ' . $checkoutId . ' | ' . $reference . '");</script>';

			return $html;
		}

		public function getPages( $title = false, $indent = true ) {
			$wp_pages = get_pages( 'sort_column=menu_order' );
			$page_list = array();
			if ( $title ) $page_list[] = $title;
			foreach ( $wp_pages as $page ) {
				$prefix = '';
				if ( $indent ) {
					$has_parent = $page->post_parent;
					while( $has_parent ) {
						$prefix .=  ' - ';
						$next_page = get_page( $has_parent );
						$has_parent = $next_page->post_parent;
					}
				}
				$page_list[$page->ID] = $prefix . $page->post_title;
			}
			return $page_list;
		}

		public function payment_fields( ) {
			if ( !empty($this->description) ) {
				echo wpautop( wptexturize( $this->description ) );
			}

			/*$order_total = $this->get_order_total();
			$order_id = $this->get_order_id();
			$currency = $this->getCurrencyFromOrder();
			$billingCountry = $this->getBillingCountry();
			$paymentLocale = $this->dataService->getPaymentLocale();*/

			echo "<script>
					var paymentMethod = getQueryVariable('payment-method');
					if (document.getElementById('payment_method_' + paymentMethod)) {
						document.getElementById('payment_method_' + paymentMethod).checked = true
						document.getElementById('payment_method_' + paymentMethod).click()
					}

					console.log('WC Payment Method', paymentMethod);

					function getQueryVariable(variable) {
						var query = window.location.search.substring(1);
						var vars = query.split('&');
						for (var i=0;i<vars.length;i++) {
							var pair = vars[i].split('=');
							if (pair[0] == variable) {
							return pair[1];
							}
						}
					}
				</script>";
		}

		// TODO
		public function showMessage( $content ) {
			$html  = 'OPPWA Plugin Message:';
			$html .= '<div class="box '.$this->msg['class'].'-box">';
			$html .= $this->msg['message'];
			$html .= '</div>';
			$html .= $content;

			return $html;
		}

		/**
		 * Output for the order received page.
		 *
		 * @param int $order_id Order ID.
		 */
		public function thankyou_page( $order_id ) {
			if ( $this->instructions ) {
				echo wp_kses_post( wpautop( wptexturize( wp_kses_post( $this->instructions ) ) ) );
			}
		}

		/**
		 * Add content to the WC emails.
		 *
		 * @param WC_Order $order Order object.
		 * @param bool     $sent_to_admin Sent to admin.
		 * @param bool     $plain_text Email format: plain text or HTML.
		 */
		public function email_instructions( $order, $sent_to_admin, $plain_text = false ) {
			if ( ! $sent_to_admin && 'oppwa' === $order->get_payment_method() && $order->has_status( 'on-hold' ) ) {
				if ( $this->instructions ) {
					echo wp_kses_post( wpautop( wptexturize( $this->instructions ) ) . PHP_EOL );
				}
			}
		}

		/**
		 * Process the payment and return the result.
		 *
		 * @param int $order_id Order ID.
		 * @return array
		 */
		public function process_payment( $order_id ) {

			global $wp_rewrite;

			$order = wc_get_order( $order_id );

			if ( $order->get_total() > 0 ) {
				// Mark as pending (we're awaiting the payment).
				$order->update_status( apply_filters( 'woocommerce_oppwa_process_payment_order_status', 'pending', $order ), __( 'Awaiting OPPWA payment', 'woocommerce' ) );
			} else {
				$order->payment_complete();
				$order->add_order_note( __('OPPWA: no payment required for 0,- (free order).', 'woothemes') );
			}

			// payment failed
			//$order->update_status( apply_filters( 'woocommerce_oppwa_process_payment_order_status', 'failed', $order ), __( 'OPPWA payment failed', 'woocommerce' ) );

			if ( $wp_rewrite->permalink_structure == '' ) {
				$checkout_url = wc_get_checkout_url() . '&order-pay= ' . $order_id . '&key= ' . $order->order_key;
			} else {
				$checkout_url = wc_get_checkout_url() . '/order-pay/' . $order_id . '?key= ' . $order->order_key;
			}

			return array(
					'result' => 'success',
					'redirect' => $checkout_url
			);
		}
	}

	/**
	 * Add OPPWA Payment Method.
	 *
	 * @param array $methods Payment methods.
	 * @return array
	 */
	function woocommerce_add_api_oppwa( $methods ) {
		$methods[] = 'WC_Gateway_oppwa';
		return $methods;
	}

	add_filter( 'woocommerce_payment_gateways', 'woocommerce_add_api_oppwa' );

	/**
	 * Add OPPWA links.
	 *
	 * @param array $links Plugin links.
	 * @return array
	 */
	function oppwa_action_links( $links ) {
		return array_merge( array(
			'<a href="' . esc_url( 'https://docs.oppwa.com' ) . '">' . __( 'OPPWA Docs', 'woocommerce' ) . '</a>',
			'<a href="' . esc_url( get_admin_url() . 'admin.php?page=wc-settings&tab=checkout&section=oppwa' ) . '">' . __( 'Settings', 'woocommerce' ) . '</a>'
		), $links );
	}

	add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'oppwa_action_links' );
}

add_action( 'plugins_loaded', 'woocommerce_api_oppwa_init', 0 );
